"""Tests for enhanced shell command tool with process management."""

import asyncio
import os
import platform
import tempfile
from pathlib import Path

import pytest

from ocode_python.tools.shell_tools_enhanced import EnhancedShellCommandTool


@pytest.mark.unit
class TestEnhancedShellCommandTool:
    """Test EnhancedShellCommandTool functionality."""

    @pytest.mark.asyncio
    async def test_basic_command_execution(self):
        """Test basic command execution."""
        tool = EnhancedShellCommandTool()

        # Simple echo command
        result = await tool.execute(command="echo 'Hello, World!'", capture_output=True)

        assert result.success
        assert "Hello, World!" in result.output
        assert result.metadata["return_code"] == 0

    @pytest.mark.asyncio
    async def test_command_with_error(self):
        """Test command that returns error."""
        tool = EnhancedShellCommandTool()

        # Command that will fail
        if platform.system() == "Windows":
            cmd = "dir /invalid_flag"
        else:
            cmd = "ls --invalid-option"

        result = await tool.execute(command=cmd, capture_output=True)

        assert not result.success
        assert result.metadata["return_code"] != 0
        assert result.error

    @pytest.mark.asyncio
    async def test_working_directory(self, tmp_path):
        """Test command execution in specific directory."""
        tool = EnhancedShellCommandTool()

        # Create test file in tmp directory
        test_file = tmp_path / "test.txt"
        test_file.write_text("test content")

        # List files in the directory
        if platform.system() == "Windows":
            cmd = "dir /b"
        else:
            cmd = "ls"

        result = await tool.execute(
            command=cmd, working_dir=str(tmp_path), capture_output=True
        )

        assert result.success
        assert "test.txt" in result.output

    @pytest.mark.asyncio
    async def test_environment_variables(self):
        """Test command with custom environment variables."""
        tool = EnhancedShellCommandTool()

        # Command to print environment variable
        if platform.system() == "Windows":
            cmd = "echo %TEST_VAR%"
        else:
            cmd = "echo $TEST_VAR"

        result = await tool.execute(
            command=cmd, env_vars={"TEST_VAR": "test_value"}, capture_output=True
        )

        assert result.success
        assert "test_value" in result.output

    @pytest.mark.asyncio
    async def test_timeout_handling(self):
        """Test command timeout."""
        tool = EnhancedShellCommandTool()

        # Command that sleeps
        if platform.system() == "Windows":
            cmd = "ping -n 5 127.0.0.1 >nul"
        else:
            cmd = "sleep 5"

        result = await tool.execute(
            command=cmd, timeout=1, capture_output=True  # 1 second timeout
        )

        assert not result.success
        assert "timed out" in result.error.lower()

    @pytest.mark.asyncio
    async def test_large_output_handling(self):
        """Test handling of large command output."""
        tool = EnhancedShellCommandTool()

        # Generate large output using a safe command
        if platform.system() == "Windows":
            # Use for loop to generate output
            cmd = "for /L %i in (1,1,10000) do @echo This is line %i"
        else:
            # Use seq to generate many lines
            cmd = "seq 1 10000"

        result = await tool.execute(
            command=cmd,
            capture_output=True,
            max_output_size=1,  # 1MB limit
            confirmed=True,
        )

        assert result.success
        # Output should be captured up to the limit
        assert len(result.output) > 0

    @pytest.mark.asyncio
    async def test_output_truncation(self):
        """Test output truncation when exceeding limit."""
        tool = EnhancedShellCommandTool()

        # Generate output larger than limit using safe command
        if platform.system() == "Windows":
            cmd = (
                "for /L %i in (1,1,100000) do @echo This is a very long line "
                "that will exceed the output limit when repeated many times"
            )
        else:
            # Use yes command with timeout to generate lots of output
            cmd = (
                "yes 'This is a very long line that will exceed the output "
                "limit when repeated many times' | head -n 100000"
            )

        result = await tool.execute(
            command=cmd,
            capture_output=True,
            max_output_size=1,  # 1MB limit
            confirmed=True,
        )

        assert result.success
        # Check for truncation message
        assert "truncated" in result.output.lower()

    @pytest.mark.asyncio
    async def test_no_capture_mode(self):
        """Test execution without output capture."""
        tool = EnhancedShellCommandTool()

        # Simple command
        result = await tool.execute(command="echo 'test'", capture_output=False)

        assert result.success
        # Output should be empty in no-capture mode
        assert result.output == ""

    @pytest.mark.asyncio
    async def test_security_validation(self):
        """Test security validation blocks dangerous commands."""
        tool = EnhancedShellCommandTool()

        dangerous_commands = [
            "sudo rm -rf /",
            "apt install malware",
            "echo evil > /etc/passwd",
        ]

        for cmd in dangerous_commands:
            result = await tool.execute(command=cmd)
            assert not result.success
            assert "blocked" in result.error.lower()

    @pytest.mark.asyncio
    async def test_confirmation_required(self):
        """Test commands requiring confirmation."""
        tool = EnhancedShellCommandTool()

        # Command that might require confirmation
        result = await tool.execute(command="curl http://example.com", confirmed=False)

        # Should either succeed or require confirmation
        if not result.success and result.error == "confirmation_required":
            assert result.metadata["requires_confirmation"]
            assert result.metadata["command"] == "curl http://example.com"

            # Retry with confirmation
            result = await tool.execute(
                command="curl http://example.com", confirmed=True
            )

    @pytest.mark.asyncio
    async def test_nonexistent_working_dir(self):
        """Test error handling for nonexistent working directory."""
        tool = EnhancedShellCommandTool()

        result = await tool.execute(
            command="echo test", working_dir="/nonexistent/directory/path"
        )

        assert not result.success
        assert "does not exist" in result.error

    @pytest.mark.asyncio
    async def test_process_cleanup_on_timeout(self):
        """Test that processes are properly cleaned up on timeout."""
        tool = EnhancedShellCommandTool()

        # Start a command that takes a long time
        if platform.system() == "Windows":
            # Windows command that sleeps
            cmd = "ping -n 10 127.0.0.1 >nul"
        else:
            # Unix command that sleeps
            cmd = "sleep 10"

        result = await tool.execute(
            command=cmd, timeout=1, kill_timeout=2, confirmed=True
        )

        assert not result.success
        assert "timed out" in result.error.lower()

        # Give time for cleanup
        await asyncio.sleep(0.5)

    @pytest.mark.asyncio
    @pytest.mark.skipif(
        platform.system() == "Windows",
        reason="Resource limits not supported on Windows",
    )
    async def test_resource_limits(self):
        """Test resource limits (Unix only)."""
        tool = EnhancedShellCommandTool()

        # Command that uses CPU
        result = await tool.execute(
            command="python3 -c 'while True: pass'",
            cpu_limit=1,  # 1 second CPU time
            timeout=5,
        )

        assert not result.success
        # Process should be killed by CPU limit

    @pytest.mark.asyncio
    async def test_stderr_capture(self):
        """Test stderr capture."""
        tool = EnhancedShellCommandTool()

        # Command that writes to stderr using a safe command
        if platform.system() == "Windows":
            cmd = "echo Error message 1>&2"
        else:
            cmd = "echo 'Error message' >&2"

        result = await tool.execute(command=cmd, capture_output=True, confirmed=True)

        assert result.success
        assert "Error message" in result.metadata.get("stderr", "")

    @pytest.mark.asyncio
    async def test_execution_time_tracking(self):
        """Test that execution time is tracked."""
        tool = EnhancedShellCommandTool()

        # Quick command
        result = await tool.execute(command="echo test", capture_output=True)

        assert result.success
        assert "execution_time" in result.metadata
        # On Windows, very fast commands might have 0.0 execution time due to
        # timer precision
        assert result.metadata["execution_time"] >= 0
        assert result.metadata["execution_time"] < 5  # Should be quick

    @pytest.mark.asyncio
    async def test_unicode_handling(self):
        """Test handling of unicode in command output."""
        tool = EnhancedShellCommandTool()

        # Command with unicode output using echo
        if platform.system() == "Windows":
            cmd = "echo Hello World"  # Simpler test without unicode
        else:
            cmd = "echo 'Hello World'"  # Simpler test without unicode

        result = await tool.execute(command=cmd, capture_output=True, confirmed=True)

        assert result.success
        assert "Hello" in result.output
        # Unicode handling tested implicitly through other tests

    @pytest.mark.asyncio
    async def test_empty_command(self):
        """Test handling of empty command."""
        tool = EnhancedShellCommandTool()

        result = await tool.execute(command="")

        assert not result.success
        assert "required" in result.error.lower()

    @pytest.mark.asyncio
    async def test_command_with_pipes(self):
        """Test command with pipes."""
        tool = EnhancedShellCommandTool()

        if platform.system() == "Windows":
            cmd = 'echo hello | find "ell"'
        else:
            cmd = "echo hello | grep ell"

        result = await tool.execute(
            command=cmd,
            capture_output=True,
            confirmed=True,  # Pipes require confirmation
        )

        assert result.success
        assert "ell" in result.output
